@@ -5,7 +5,7 @@
  * Massachusetts.
  *
  * Enhancements Copyright 1992-2001, 2002, 2003, 2004, 2005, 2006,
- * 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Free Software Foundation, Inc.
+ * 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016 Free Software Foundation, Inc.
  *
  * Enhancements Copyright 2005 Alessandro Scotti
  *
@@ -100,10 +100,11 @@ typedef struct {
 } IcsTextMenuEntry;
 
 IcsTextMenuEntry icsTextMenuEntry[ICS_TEXT_MENU_SIZE];
+char dataDir[MSG_SIZ] = DATADIR;
 
 int junk;
-unsigned int saveDate;
-unsigned int dateStamp;
+int saveDate;
+int dateStamp;
 Boolean singleList;
 Boolean autoClose;
 char *homeDir;
@@ -222,7 +223,8 @@ ArgDescriptor argDescriptors[] = {
   { "flashRate", ArgInt, (void *) &appData.flashRate, XBOARD, (ArgIniType) FLASH_RATE },
   { "pieceImageDirectory", ArgFilename, (void *) &appData.pieceDirectory, TRUE, (ArgIniType) "" },
   { "pid", ArgFilename, (void *) &appData.pieceDirectory, FALSE, INVALID },
-  { "trueColors", ArgBoolean, (void *) &appData.trueColors, TRUE, (ArgIniType) FALSE },
+  { "trueColors", ArgBoolean, (void *) &appData.trueColors, XBOARD, (ArgIniType) (2*!XBOARD) },
+  { "jewelled", ArgInt, (void *) &appData.jewelled, FALSE, (ArgIniType) -1 },
   { "soundDirectory", ArgFilename, (void *) &appData.soundDirectory, XBOARD, (ArgIniType) "" },
   { "msLoginDelay", ArgInt, (void *) &appData.msLoginDelay, XBOARD, (ArgIniType) MS_LOGIN_DELAY },
   { "pasteSelection", ArgBoolean, (void *) &appData.pasteSelection, XBOARD, (ArgIniType) FALSE },
@@ -230,6 +232,7 @@ ArgDescriptor argDescriptors[] = {
   { "dropMenu", ArgBoolean, (void *) &appData.dropMenu, TRUE, (ArgIniType) FALSE },
   { "pieceMenu", ArgBoolean, (void *) &appData.pieceMenu, TRUE, (ArgIniType) TRUE },
   { "sweepPromotions", ArgBoolean, (void *) &appData.sweepSelect, TRUE, (ArgIniType) FALSE },
+  { "monoMouse", ArgBoolean, (void *) &appData.monoMouse, XBOARD, (ArgIniType) FALSE },
   { "remoteShell", ArgFilename, (void *) &appData.remoteShell, FALSE, (ArgIniType) REMOTE_SHELL },
   { "rsh", ArgFilename, (void *) &appData.remoteShell, FALSE, INVALID },
   { "remoteUser", ArgString, (void *) &appData.remoteUser, FALSE, (ArgIniType) "" },
@@ -283,6 +286,7 @@ ArgDescriptor argDescriptors[] = {
   { "lpf", ArgFilename, (void *) &appData.loadPositionFile, FALSE, INVALID },
   { "loadPositionIndex", ArgInt, (void *) &appData.loadPositionIndex, FALSE, (ArgIniType) 1 },
   { "lpi", ArgInt, (void *) &appData.loadPositionIndex, FALSE, INVALID },
+  { "positionDir", ArgFilename, (void *) &appData.positionDir, FALSE, (ArgIniType) "" },
   { "savePositionFile", ArgFilename, (void *) &appData.savePositionFile, FALSE, (ArgIniType) "" },
   { "spf", ArgFilename, (void *) &appData.savePositionFile, FALSE, INVALID },
   { "matchMode", ArgBoolean, (void *) &appData.matchMode, FALSE, (ArgIniType) FALSE },
@@ -515,6 +519,7 @@ ArgDescriptor argDescriptors[] = {
   { "themeNames", ArgString, (void *) &appData.themeNames, TRUE, (ArgIniType) "native -upf false -ub false -ubt false -pid \"\"\n" },
   { "addMasterOption", ArgMaster, NULL, FALSE, INVALID },
   { "installEngine", ArgInstall, (void *) &firstChessProgramNames, FALSE, (ArgIniType) "" },
+  { "installTheme", ArgInstall, (void *) &appData.themeNames, FALSE, (ArgIniType) "" },
   { "initialMode", ArgString, (void *) &appData.initialMode, FALSE, (ArgIniType) "" },
   { "mode", ArgString, (void *) &appData.initialMode, FALSE, INVALID },
   { "variant", ArgString, (void *) &appData.variant, FALSE, (ArgIniType) "normal" },
@@ -530,6 +535,7 @@ ArgDescriptor argDescriptors[] = {
   { "secondScoreAbs", ArgBoolean, (void *) &appData.secondScoreIsAbsolute, FALSE, (ArgIniType) FALSE },
   { "pgnExtendedInfo", ArgBoolean, (void *) &appData.saveExtendedInfoInPGN, TRUE, (ArgIniType) FALSE },
   { "hideThinkingFromHuman", ArgBoolean, (void *) &appData.hideThinkingFromHuman, TRUE, (ArgIniType) FALSE },
+  { "pgnTimeLeft", ArgBoolean, (void *) &appData.cumulativeTimePGN, TRUE, (ArgIniType) FALSE },
   { "liteBackTextureFile", ArgFilename, (void *) &appData.liteBackTextureFile, TRUE, (ArgIniType) "" },
   { "lbtf", ArgFilename, (void *) &appData.liteBackTextureFile, FALSE, INVALID },
   { "darkBackTextureFile", ArgFilename, (void *) &appData.darkBackTextureFile, TRUE, (ArgIniType) "" },
@@ -608,9 +614,11 @@ ArgDescriptor argDescriptors[] = {
   { "ub", ArgBoolean, (void *) &appData.useBorder, FALSE, INVALID },
   { "border", ArgFilename, (void *) &appData.border, TRUE, (ArgIniType) "" },
   { "finger", ArgFilename, (void *) &appData.finger, FALSE, (ArgIniType) "" },
-  { "inscriptions", ArgString, (void *) &appData.inscriptions, XBOARD, (ArgIniType) "" },
+  { "epd", ArgTrue, (void *) &appData.epd, FALSE, INVALID },
+  { "inscriptions", ArgString, (void *) &appData.inscriptions, FALSE, (ArgIniType) "" },
   { "autoInstall", ArgString, (void *) &appData.autoInstall, XBOARD, (ArgIniType) "" },
   { "fixedSize", ArgBoolean, (void *) &appData.fixedSize, TRUE, (ArgIniType) FALSE },
+  { "showMoveTime", ArgBoolean, (void *) &appData.moveTime, TRUE, (ArgIniType) FALSE },
 
   // [HGM] tournament options
   { "tourneyFile", ArgFilename, (void *) &appData.tourneyFile, FALSE, (ArgIniType) "" },
@@ -632,6 +640,7 @@ ArgDescriptor argDescriptors[] = {
   { "oneClickMove", ArgBoolean, (void *) &appData.oneClick, TRUE, (ArgIniType) FALSE },
   { "boardWidth", ArgInt, (void *) &appData.NrFiles, FALSE, (ArgIniType) -1 },
   { "boardHeight", ArgInt, (void *) &appData.NrRanks, FALSE, (ArgIniType) -1 },
+  { "rankOffset", ArgInt, (void *) &appData.rankOffset, FALSE, (ArgIniType) 0 },
   { "holdingsSize", ArgInt, (void *) &appData.holdingsSize, FALSE, (ArgIniType) -1 },
   { "defaultMatchGames", ArgInt, (void *) &appData.defaultMatchGames, TRUE, (ArgIniType) 10 },
   { "matchPause", ArgInt, (void *) &appData.matchPause, TRUE, (ArgIniType) 10000 },
@@ -706,6 +715,11 @@ ArgDescriptor argDescriptors[] = {
   { "firstDrawDepth", ArgInt, (void *) &appData.drawDepth[0], FALSE, (ArgIniType) 0 },
   { "secondDrawDepth", ArgInt, (void *) &appData.drawDepth[1], FALSE, (ArgIniType) 0 },
   { "memoHeaders", ArgBoolean, (void *) &appData.headers, TRUE, (ArgIniType) FALSE },
+  { "startupMessage", ArgString, (void *) &appData.message, FALSE, (ArgIniType) "" },
+  { "messageSuppress", ArgString, (void *) &appData.suppress, XBOARD, (ArgIniType) "" },
+  { "fen", ArgString, (void *) &appData.fen, FALSE, (ArgIniType) "" },
+  { "men", ArgString, (void *) &appData.men, FALSE, (ArgIniType) "" },
+  { "analysisBell", ArgInt, (void *) &appData.analysisBell, TRUE, (ArgIniType) 0 },
 
 #if ZIPPY
   { "zippyTalk", ArgBoolean, (void *) &appData.zippyTalk, FALSE, (ArgIniType) ZIPPY_TALK },
@@ -924,7 +938,7 @@ ParseSettingsFile(char *name, char **addr)
 	MySearchPath(installDir, buf, fullname); // first look in user's own files
 	f = fopen(fullname, "r");
 	if(f == NULL) {
-	    snprintf(buf, MSG_SIZ, "%s/themes/conf", DATADIR);
+	    snprintf(buf, MSG_SIZ, "%s/themes/conf", dataDir);
 	    MySearchPath(buf, name, fullname); // also look in standard place
 	    f = fopen(fullname, "r");
 	}
@@ -949,7 +963,7 @@ ParseArgs(GetFunc get, void *cl)
   char argValue[MAX_ARG_LEN];
   ArgDescriptor *ad;
   char start;
-  char *q;
+  char *q, *r, *s;
   int i, octval;
   char ch;
   int posarg = 4; // default is game file
@@ -1164,7 +1178,7 @@ ParseArgs(GetFunc get, void *cl)
     case ArgFilename:
       if(argValue[0] == '~' && argValue[1] == '~') {
         char buf[4*MSG_SIZ]; // expand ~~
-        snprintf(buf, 4*MSG_SIZ, "%s%s", DATADIR, argValue+2);
+        snprintf(buf, 4*MSG_SIZ, "%s%s", dataDir, argValue+2);
         ASSIGN(*(char **) ad->argLoc, buf);
         break;
       }
@@ -1229,10 +1243,29 @@ ParseArgs(GetFunc get, void *cl)
 
     case ArgInstall:
       q = *(char **) ad->argLoc;
-      if((saveDate == 0 || saveDate - dateStamp < 0) && !strstr(q, argValue) ) {
-        int l = strlen(q) + strlen(argValue);
-        *(char **) ad->argLoc = malloc(l+2);
-        snprintf(*(char **) ad->argLoc, l+2, "%s%s\n", q, argValue);
+      r = NULL; s = argValue;
+      if(argValue[0] == '#') { // group specification
+        r = strstr(argValue, "\\n");
+        if(r) *r++ = '\n', *r++ = NULLCHAR, s = r, r = argValue; // split s into line-to-add (s) and group (r)
+      }
+      if((saveDate == 0 || saveDate - dateStamp < 0) && !strstr(q, s) ) { // not seen before, and line does not occur yet
+        int l = strlen(q) + strlen(s);
+        if(r) { // must be put in group r
+          char *p = strstr(q, r);
+          if(p) { // group already exists
+            p += strlen(r) - 1; // determine insertion point (immediately after group header line)
+            *(char **) ad->argLoc = malloc(l+2);
+            *p++ = NULLCHAR; // spit old value (q) at insertion point into q and p
+            snprintf(*(char **) ad->argLoc, l+2, "%s\n%s\n%s", q, s, p); // insert (with newline)
+          } else { // group did not exist, create at end
+            l += strlen(r) + 8;
+            *(char **) ad->argLoc = malloc(l);
+            snprintf(*(char **) ad->argLoc, l, "%s%s%s\n# end\n", q, r, s);
+          }
+        } else { // no group, just add line at end
+          *(char **) ad->argLoc = malloc(l+2);
+          snprintf(*(char **) ad->argLoc, l+2, "%s%s\n", q, s);
+        }
         free(q);
       }
       break;
@@ -1405,7 +1438,11 @@ InitAppData(char *lpCmdLine)
      appData.NrRanks > BOARD_RANKS   )
       DisplayFatalError("Recompile with BOARD_RANKS or BOARD_FILES, to support this size", 0, 2);
 
-  if(!*appData.secondChessProgram) { ASSIGN(appData.secondChessProgram, appData.firstChessProgram); } // [HGM] scp defaults to fcp
+  if(!*appData.secondChessProgram) { // [HGM] scp defaults to fcp
+    ASSIGN(appData.secondChessProgram, appData.firstChessProgram);
+    ASSIGN(appData.secondDirectory, appData.firstDirectory);
+    appData.secondIsUCI = appData.firstIsUCI; // copy type too!
+  }
 
   /* [HGM] After parsing the options from the .ini file, and overruling them
    * with options from the command line, we now make an even higher priority
@@ -1458,6 +1495,11 @@ InitAppData(char *lpCmdLine)
     } else appData.timeIncrement = -1;
   }
   if(appData.movesPerSession <= 0) appData.movesPerSession = MOVES_PER_SESSION; // mps <= 0 is invalid in any case
+  if(*appData.defaultPathEGTB) { // append value of deprecated -defaultPathEGTB to -egtFormats
+    snprintf(buf, MAX_ARG_LEN, "%s%snalimov:%s", appData.egtFormats, (*appData.egtFormats ?"," : ""), appData.defaultPathEGTB);
+    ASSIGN(appData.egtFormats, buf);
+    ASSIGN(appData.defaultPathEGTB, "");
+  }
 
   /* Open startup dialog if needed */
   if ((!appData.noChessProgram && !chessProgram && !appData.icsActive) ||
@@ -1509,11 +1551,10 @@ SaveSettings(char* name)
   ArgDescriptor *ad;
   char dir[MSG_SIZ], buf[MSG_SIZ];
   int mps = appData.movesPerSession;
-  TimeMark now;
 
   if (!MainWindowUp() && !autoClose) return;
 
-  GetTimeMark(&now); saveDate = now.sec;
+  saveDate = programStartTime.sec;
 
   GetCurrentDirectory(MSG_SIZ, dir);
   if(MySearchPath(installDir, name, buf)) {
@@ -1616,8 +1657,8 @@ SaveSettings(char* name)
       { char buf[MSG_SIZ];
         snprintf(buf, MSG_SIZ, "%s", *(char**)ad->argLoc);
 #ifdef OSXAPP
-        if(strstr(buf, DATADIR) == buf)
-          snprintf(buf, MSG_SIZ, "~~%s", *(char**)ad->argLoc + strlen(DATADIR));
+        if(strstr(buf, dataDir) == buf)
+          snprintf(buf, MSG_SIZ, "~~%s", *(char**)ad->argLoc + strlen(dataDir));
 #endif
         if (strchr(buf, '\"')) {
           fprintf(f, OPTCHAR "%s" SEPCHAR "'%s'\n", ad->argName, buf);
