@@ -5,7 +5,8 @@
  * Massachusetts.
  *
  * Enhancements Copyright 1992-2001, 2002, 2003, 2004, 2005, 2006,
- * 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014 Free Software Foundation, Inc.
+ * 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016 Free
+ * Software Foundation, Inc.
  *
  * Enhancements Copyright 2005 Alessandro Scotti
  *
@@ -79,6 +80,7 @@ extern Boolean mcMode;
 extern int dragging;
 extern char variantError[];
 extern char lastTalker[];
+extern int transparency[];
 
 void MarkMenuItem P((char *menuRef, int state));
 char *CmailMsg P((void));
@@ -217,6 +219,8 @@ void ics_printf P((char *format, ...));
 int GetEngineLine P((char *nick, int engine));
 void AddGameToBook P((int always));
 void FlushBook P((void));
+char PieceToChar P((ChessSquare p));
+int LoadPieceDesc P((char *s));
 
 char *StrStr P((char *string, char *match));
 char *StrCaseStr P((char *string, char *match));
@@ -416,13 +420,17 @@ char *EngineDefinedVariant P((ChessProgramState *cps, int n));
 void SettingsPopUp P((ChessProgramState *cps)); // [HGM] really in front-end, but CPS not known in frontend.h
 int WaitForEngine P((ChessProgramState *cps, DelayedEventCallback x));
 void Load P((ChessProgramState *cps, int n));
-int MultiPV P((ChessProgramState *cps));
+int MultiPV P((ChessProgramState *cps, int kind));
 void MoveHistorySet P(( char movelist[][2*MOVE_LEN], int first, int last, int current, ChessProgramStats_Move * pvInfo ));
 void MakeEngineOutputTitle P((void));
 void LoadTheme P((void));
 void CreateBookEvent P((void));
 char *SupportedVariant P((char *list, VariantClass v, int w, int h, int s, int proto, char *engine));
 char *CollectPieceDescriptors P((void));
+void RefreshSettingsDialog P((ChessProgramState *cps, int val));
+void StartChessProgram P((ChessProgramState *cps));
+void SendToICS P((char *s));
+int PosFlags P((int n));
 
 
 /* A point in time */
